<?php

namespace RESTful;

use RESTful\RESTfulInterface;

class RESTful implements RESTfulInterface 
{
    private $_baseURI;
    private $_timeout;
    private $_token;

    /**
     * Constructor
     *
     * @param String $baseURI Base URI
     * @param String $token token 
     *
     * @return void
     */
    function __construct($baseURI = '', $token = '') 
    {
        if ($baseURI == '') {
            trigger_error('You must provide a baseURI variable.', E_USER_ERROR);
        }

        if (!extension_loaded('curl')) {
            trigger_error('Extension CURL is not loaded.', E_USER_ERROR);
        }

        $this->setBaseURI($baseURI);
        $this->setTimeOut(10);
        $this->setToken($token);
    }

    /**
     * Sets Token
     *
     * @param String $token Token
     *
     * @return void
     */
    public function setToken($token) 
    {
        $this->_token = $token;
    }

    /**
       * Sets Base URI, ex: http://yourcompany.com/<path>/<to>/<resource>
     *
     * @param String $baseURI Base URI
     *
     * @return void
     */
    public function setBaseURI($baseURI) 
    {
        $baseURI .= (substr($baseURI, -1) == '/' ? '' : '/');
        $this->_baseURI = $baseURI;
    }

    /**
     * Returns with the normalized JSON absolute path
     *
     * @param String $path to data
     */
    private function _getJsonPath($path) 
    {
        $url = $this->_baseURI;
        $path = ltrim($path, '/');
        $auth = ($this->_token == '') ? '' : '?auth=' . $this->_token;
        return $url . $path . '.json' . $auth;
    }

    /**
     * Sets REST call timeout in seconds
     *
     * @param Integer $seconds Seconds to timeout
     *
     * @return void
     */
    public function setTimeOut($seconds) 
    {
        $this->_timeout = $seconds;
    }

    /**
     * Writing data into RESTful with a PUT request
     * HTTP 200: Ok
     *
     * @param String $path Path
     * @param Mixed  $data Data
     *
     * @return String Response
     */
    public function put($path, $data) 
    {
      return $this->_writeData($path, $data, 'PUT');
    }

    /**
     * Pushing data into RESTful with a POST request
     * HTTP 200: Ok
     *
     * @param String $path Path
     * @param Mixed  $data Data
     *
     * @return String Response
     */
    public function post($path, $data) 
    {
      return $this->_writeData($path, $data, 'POST');
    }

    /**
     * Updating data into RESTful with a PATH request
     * HTTP 200: Ok
     *
     * @param String $path Path
     * @param Mixed  $data Data
     *
     * @return String Response
     */
    public function patch($path, $data) 
    {
      return $this->_writeData($path, $data, 'PATCH');
    }

    /**
     * Reading data from RESTful
     * HTTP 200: Ok
     *
     * @param String $path Path
     *
     * @return String Response
     */
    public function get($path) 
    {
        try {
            $ch = $this->_getCurlHandler($path, 'GET');
            $return = curl_exec($ch);
            curl_close($ch);
        } catch (\Exception $e) {
            $return = null;
        }
        return $return;
    }

    /**
     * Deletes data from RESTful
     * HTTP 204: Ok
     *
     * @param type $path Path
     *
     * @return String Response
     */
    public function delete($path) 
    {
        try {
            $ch = $this->_getCurlHandler($path, 'DELETE');
            $return = curl_exec($ch);
            curl_close($ch);
        } catch (\Exception $e) {
            $return = null;
        }
        return $return;
    }

    /**
     * Returns with Initialized CURL Handler
     *
     * @param String $mode Mode
     *
     * @return CURL Curl Handler
     */
    private function _getCurlHandler($path, $mode) 
    {
        $url = $this->_getJsonPath($path);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->_timeout);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $this->_timeout);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $mode);
        return $ch;
    }

    private function _writeData($path, $data, $method = 'PUT') 
    {
        $jsonData = json_encode($data);
        $header = array(
            'Content-Type: application/json',
            'Content-Length: ' . strlen($jsonData)
        );
        try {
            $ch = $this->_getCurlHandler($path, $method);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
            $return = curl_exec($ch);
            curl_close($ch);
        } catch (\Exception $e) {
            $return = null;
        }
        return $return;
    }
}
